---
layout: default
title:  "Other interesting use cases"
date:   2017-07-23 9:40:48 +0300
categories: [abilities]
tags: [feature flags, hardware capabilities]
---

The primary goal of CASL is to provide simple way to declare user abilities in application. In the same time application may or may not have some capabilities depending on hardware, user's culture, subscription plan, etc. Lets see how these cases can be covered by CASL.

## Hardware capabilities

Imagine that you write software for embed devices (e.g., cars, chipsets, management system for servers, etc). Using CASL you can define what user can do
in your application based on device capabilities:

```js
const { AbilityBuilder } = require('casl')
const capabilities = require('device-capabilities')

const ability = AbilityBuilder.define(can => {
  if (capabilities.version >= 10) {
    can('manage', 'wifi')
  }

  if (capabilities.supports('audio')) {
    can('manage', 'audio')
  }

  can('manage', 'ethernet')
})
```

And later in UI check the capability with help of `ability.can('read', 'wifi')`. You can mix these definitions with your permission logic if you need. In this way you have 2 levels of permission: device level (whether such capability exists at all) and role/subscription level (whether a user has the right to do this). But be careful with the order order of abilities. See [Combining Abilities][combining-abilities] for details.

## Feature flags

Also you can use CASL to define feature flags and run [A/B testing](https://en.wikipedia.org/wiki/A/B_testing) for new features.

```js
const { AbilityBuilder } = require('casl')

function defineAbilitiesFor(user) {
  return AbilityBuilder.define(can => {
    if (user.age >= 18) {
      can('manage', 'Event')
    } else {
      can('read', 'Event')
    }
  })  
}
```

[combining-abilities]: {% post_url 2017-07-20-define-abilities %}#combining-abilities
